@@ -24,10 +24,6 @@ print.summary.SemiParBIVProbit <- function(x, digits = max(3, getOption("digits"
   if(x$sel==FALSE) cat("\n\nEQUATION 1") else cat("\n\nSELECTION EQ.") 
   if(x$PL=="P") cat("\nLink function: probit\n")
 
-  #if(x$PL=="PP") cat("\nLink function: power probit, ",format(x$xi1,digits=3),"(",format(x$CIl1[1],digits=3),",",format(x$CIl1[2],digits=3),")\n", sep="") 
-  #if(x$PL=="RPP") cat("\nLink function: reciprocal power probit, ",format(x$lambda1,digits=3),"(",format(x$CIl1[1],digits=3),",",format(x$CIl1[2],digits=3),")\n",sep="") 
-
-
   if(x$PL=="PP" || x$PL=="RPP"){ 
     if(round(x$xi1,3)==1) cat("\nLink function: probit\n") else{ 
   	if(x$PL=="PP") cat("\nLink function: power probit, ",format(x$xi1,digits=3),"\n", sep="") 
@@ -60,9 +56,6 @@ print.summary.SemiParBIVProbit <- function(x, digits = max(3, getOption("digits"
   if(x$sel==FALSE) cat("\nEQUATION 2") else cat("\nOUTCOME EQ.")
   if(x$PL=="P") cat("\nLink function: probit\n")
 
-  #if(x$PL=="PP") cat("\nLink function: power probit, ",format(x$xi2,digits=3),"(",format(x$CIl2[1],digits=3),",",format(x$CIl2[2],digits=3),")\n", sep="") 
-  #if(x$PL=="RPP") cat("\nLink function: reciprocal power probit, ",format(x$xi2,digits=3),"(",format(x$CIl2[1],digits=3),",",format(x$CIl2[2],digits=3),")\n",sep="") 
-
 
   if(x$PL=="PP" || x$PL=="RPP"){ 
     if(round(x$xi2,3)==1) cat("\nLink function: probit\n") else{  
@@ -96,13 +89,31 @@ print.summary.SemiParBIVProbit <- function(x, digits = max(3, getOption("digits"
   if(x$BivD %in% c("N","T")) {cp <- "  rho = "; as.p <- x$rho} else{ cp <- "  theta = "; as.p <- x$theta}
 
 
-  if(x$Model=="B" ) cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","  Kendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","\ntotal edf = ",format(x$t.edf,digits=3),"  MR = ",format(x$MR,digits=3),"%","  QPS1 = ",format(x$QPS1,digits=3),"  QPS2 = ",format(x$QPS2,digits=3),"\nCR1 = ",format(x$CR1,digits=3),"%  CR2 = ",format(x$CR2,digits=3),"%\n\n", sep="")  
+  #if(x$Model=="B" )  cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","  total edf = ",format(x$t.edf,digits=3),"\nMR = ",format(x$MR,digits=3),"%","  QPS1 = ",format(x$QPS1,digits=3),"  QPS2 = ",format(x$QPS2,digits=3),"\nCR1 = ",format(x$CR1,digits=3),"%  CR2 = ",format(x$CR2,digits=3),"%\n\n", sep="")     
+  #if(x$Model=="BSS") cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\ntotal edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+  #if(x$Model=="BPO") cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+
+       
+  
+  #if(x$Model=="B" ) cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","  Kendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","\ntotal edf = ",format(x$t.edf,digits=3),"  MR = ",format(x$MR,digits=3),"%","  QPS1 = ",format(x$QPS1,digits=3),"  QPS2 = ",format(x$QPS2,digits=3),"\nCR1 = ",format(x$CR1,digits=3),"%  CR2 = ",format(x$CR2,digits=3),"%\n\n", sep="")  
      
-  if(x$Model=="BSS") cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+  #if(x$Model=="BSS") cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
 
-  if(x$Model=="BPO") cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+  #if(x$Model=="BPO") cat("\nn = ",x$n,cp,format(as.p,digits=3),"(",format(x$CIrs[1],digits=3),",",format(x$CIrs[2],digits=3),")","\nKendall's Tau = ",round(x$KeT,3),"(",round(x$CIkt[1],3),",",round(x$CIkt[2],3),")","  total edf = ",format(x$t.edf,digits=3),"\n\n", sep="") 
+     
+     
+     
+     
+  nodi <- 3
+  
+  if(x$Model=="B" ) cat("\nn = ",x$n,cp,format(as.p,digits=nodi),"(",format(x$CIrs[1],digits=nodi),",",format(x$CIrs[2],digits=nodi),")","  Gamma measure = ",format(x$GM,digits=nodi),"(",format(x$CIgm[1],digits=nodi),",",format(x$CIgm[2],digits=nodi),")","\ntotal edf = ",format(x$t.edf,digits=nodi),"  MR = ",format(x$MR,digits=nodi),"%","  QPS1 = ",format(x$QPS1,digits=nodi),"  QPS2 = ",format(x$QPS2,digits=nodi),"\nCR1 = ",format(x$CR1,digits=nodi),"%  CR2 = ",format(x$CR2,digits=nodi),"%\n\n", sep="")  
+     
+  if(x$Model=="BSS") cat("\nn = ",x$n,"  n.sel = ",x$n.sel,cp,format(as.p,digits=nodi),"(",format(x$CIrs[1],digits=nodi),",",format(x$CIrs[2],digits=nodi),")","\nGamma measure = ",format(x$GM,digits=nodi),"(",format(x$CIgm[1],digits=nodi),",",format(x$CIgm[2],digits=nodi),")","  total edf = ",format(x$t.edf,digits=nodi),"\n\n", sep="") 
 
-                       
+  if(x$Model=="BPO") cat("\nn = ",x$n,cp,format(as.p,digits=nodi),"(",format(x$CIrs[1],digits=nodi),",",format(x$CIrs[2],digits=nodi),")","  Gamma measure = ",format(x$GM,digits=nodi),"(",format(x$CIgm[1],digits=nodi),",",format(x$CIgm[2],digits=nodi),")","\ntotal edf = ",format(x$t.edf,digits=nodi),"\n\n", sep="") 
+            
+       
+       
 invisible(x)
                 
 }
